import * as helpers from "./modules/helpers.js";

const config = {
  controls: {
    mouse: true,
    button: true,
    voice: true,
  },
};

const navigation = {
  home: {
    label: "Home",
    path: "/",
    command: "home",
    submenu: {
      menu: {
        label: "Menu",
        path: "/menu",
        command: "menu",
        submenu: {
          call: {
            label: "Call",
            path: "/menu/call",
            command: "call",
            submenu: {
              emergency: {
                label: "Emergency",
                command: "emergency",
                path: "/menu/call/emergency",
                message: "Calling Emergency",
              },
              contact1: {
                label: "Flute Inc.",
                command: "contact A",
                path: "/menu/call/contact1",
                message: "Calling Flute Inc.",
              },
              contact2: {
                label: "Brays AG",
                command: "contact B",
                path: "/menu/call/contact2",
                message: "Calling Brays AG",
              },
            },
          },
          check: {
            label: "Check",
            path: "/menu/check",
            command: "check",
            submenu: {
              daily: {
                label: "Daily",
                command: "daily",
                path: "/menu/check/daily",
                message: "Displaying Daily Checklist",
              },
              weekly: {
                label: "Weekly",
                command: "weekly",
                path: "/menu/check/weekly",
                message: "Displaying Weekly Checklist",
              },
              monthly: {
                label: "Monthly",
                command: "monthly",
                path: "/menu/check/monthly",
                message: "Displaying Monthly Checklist",
              },
            },
          },
          apps: {
            label: "Apps",
            path: "/menu/apps",
            command: "apps",
            submenu: {
              record: {
                label: "Record",
                command: "record",
                path: "/menu/apps/record",
                message: "Opening Recording App",
              },
              breakout: {
                label: "Breakout",
                command: "breakout",
                path: "/menu/apps/breakout",
                message: "Opening Breakout App",
              },
              statistics: {
                label: "Statistics",
                command: "statistics",
                path: "/menu/apps/statistics",
                message: "Opening Statistics App",
              },
            },
          },
        },
      },
    },
  },
};

// init application
function init() {
  // build navigation
  const appEl = document.getElementById("app");
  const navigationEl = document.createElement("nav");
  const listEl = document.createElement("ul");
  navigationEl.id = "navigation";
  navigationEl.appendChild(listEl);
  appEl.appendChild(navigationEl);

  // navigate to current path or root
  const pathname = window.location.pathname;
  if (pathname) navigateToPath(pathname);
  else navigateToPath("/");

  // Add a popstate event listener to the window object
  window.addEventListener("popstate", function (event) {
    var state = event.state;

    // Load the appropriate page based on the state object
    // without pushing it to the history
    if (state) navigateToPath(state.pageUrl, false);
    else navigateToPath("/", false);
  });

  // adding controls
  if (config.controls.mouse) addMouseControls();
  if (config.controls.button) addButtonControls();
  if (config.controls.voice) addVoiceControls();
}

// update navigation
function buildPageContents(object) {
  // handle message content
  if (object.hasOwnProperty("message")) {
    // create content element
    const appEl = document.getElementById("app");
    const contentEl = document.createElement("div");
    contentEl.id = "content";
    contentEl.innerHTML = object.message;
    appEl.prepend(contentEl);
  } else {
    // remove content element
    const contentEl = document.getElementById("content");
    if (contentEl) contentEl.remove();
  }

  updateNavigation();

  // update navigation
  function updateNavigation() {
    const listEl = document.createElement("ul");

    // add navigation items
    for (let key in object.submenu) {
      let itemEl = buildNavigationItemEl(object.submenu[key]);
      itemEl.classList.add("forward-btn");
      listEl.appendChild(itemEl);
    }

    // add "special back" button
    if (object.path !== "/") {
      // determine depth of object
      const segments = object.path.split("/");
      const depth = segments.length - 1;
      let backItemEl;
      if (depth === 1)
        backItemEl = buildNavigationItemEl({
          label: "Close",
          path: "/",
          command: "close",
        });
      else if (depth === 2)
        backItemEl = buildNavigationItemEl({
          label: "Back",
          path: "/menu",
          command: "back",
        });
      else if (depth === 3)
        backItemEl = buildNavigationItemEl({
          label: "Exit",
          path: "/",
          command: "exit",
        });

      backItemEl.classList.add("back-btn");
      listEl.appendChild(backItemEl);
    }

    const navigationEl = document.getElementById("navigation");
    navigationEl.replaceChildren(listEl);

    // build navigation item
    function buildNavigationItemEl(item) {
      const itemEl = document.createElement("li");
      itemEl.className = "link";
      const linkEl = document.createElement("a");
      linkEl.href = item.path;
      linkEl.innerHTML = item.label;
      linkEl.dataset.command = item.command;
      itemEl.appendChild(linkEl);

      if (item.label.toLowerCase() !== item.command.toLowerCase()) {
        const commandEl = document.createElement("div");
        commandEl.className = "command";
        commandEl.innerHTML = item.command;
        itemEl.appendChild(commandEl);
      }

      return itemEl;
    }
  }
}

// general navigation
// ===================

// navigate to path
function navigateToPath(path, pushState = true) {
  // find object with path
  let navObj = helpers.findNestedObject(navigation, "path", path);
  if (!navObj) navObj = navigation.home; // fallback to home
  buildPageContents(navObj);

  // State Object
  var stateObj = {
    pageTitle: navObj.label,
    pageUrl: navObj.path,
  };

  // Push state to browser history
  if (pushState)
    history.pushState(stateObj, stateObj.pageTitle, stateObj.pageUrl);
}

// navigate to currently selected item
function navigateToSelected() {
  const currentItem = document.querySelector(
    "#navigation > ul > li[data-selected] a"
  );
  const path = currentItem.getAttribute("href");
  navigateToPath(path);
}

// focus specific navigation item
function focusNavigationItem(index) {
  // unselect all items
  const navigationList = document.querySelectorAll("#navigation li");
  for (let item of navigationList) delete item.dataset.selected;

  // select requested element
  const navigationItem = document.querySelector(
    `#navigation > ul > li:nth-child(${index})`
  );
  navigationItem.dataset.selected = "";
}

// focus next navigation item
function focusNextNavigationItem() {
  // get currently selected element
  let currentItem = document.querySelector(
    "#navigation > ul > li[data-selected]"
  );

  if (!currentItem) focusNavigationItem(1);
  else if (currentItem.nextSibling) {
    delete currentItem.dataset.selected;
    currentItem.nextSibling.dataset.selected = "";
  } else focusNavigationItem(1);
}

// focus previous navigation item
function focusPreviousNavigationItem() {
  // get currently selected element
  let currentItem = document.querySelector(
    "#navigation > ul > li[data-selected]"
  );

  if (!currentItem) focusNavigationItem(1);
  else if (currentItem.previousSibling) {
    delete currentItem.dataset.selected;
    currentItem.previousSibling.dataset.selected = "";
  } else {
    let childCount = document.querySelector("#navigation > ul").children.length;
    focusNavigationItem(childCount);
  }
}

// moving up navigation tree
function moveUpNavigationLevel() {
  const path = window.location.pathname;
  let parentPath = path.slice(0, path.lastIndexOf("/"));
  navigateToPath(parentPath);
}

// moving down navigation tree
function moveDownNavigationLevel() {
  const currentItem = document.querySelector(
    "#navigation > ul > li[data-selected]"
  );
  if (currentItem) {
    const isBackButton = currentItem.classList.contains("back-btn");
    if (!isBackButton) navigateToSelected();
  }
}

// add mouse controls
// ==================
function addMouseControls() {
  const appEl = document.getElementById("app");

  // focus on mouseover
  appEl.addEventListener("mouseover", (event) => {
    const listItem = event.target.closest("li.link");
    if (listItem) {
      // get childnode index
      let index = Array.from(listItem.parentNode.children).indexOf(listItem);
      focusNavigationItem(index + 1);
    }
  });

  // select on mouseclick
  appEl.addEventListener("click", (event) => {
    event.preventDefault();
    const listItem = event.target.closest("li.link");
    if (listItem) {
      const link = listItem.querySelector("a");
      const path = link.getAttribute("href");
      navigateToPath(path);
    }
  });
}

// add button controls
// ===================
function addButtonControls() {
  // add two button controls for menu
  document.addEventListener("keydown", onKeyDown);

  // on keydown event
  function onKeyDown(event) {

    printLogMsg(event.code);

    // prevent defaults
    if (
      event.code === "Tab" ||
      event.code === "Enter" ||
      event.code === "Escape" ||
      event.code === "ArrowDown" ||
      event.code === "ArrowUp" ||
      event.code === "ArrowRight"
    )
      event.preventDefault();

    // browsing navigation items
    if (event.code === "Tab" || event.code === "ArrowDown")
      focusNextNavigationItem();
    else if (event.code === "ArrowUp") focusPreviousNavigationItem();
    else if (event.code === "Enter") navigateToSelected();
    else if (event.code === "ArrowRight") moveDownNavigationLevel();
    else if (event.code === "ArrowLeft" || event.code === "Escape")
      moveUpNavigationLevel();
  }

  console.log(navigator.mediaSession);

  // listening for hardware buttons
  navigator.mediaSession.setActionHandler("previoustrack", () => {
    const logEl = document.getElementById("log");
    logEl.innerHTML += "volumeup";
  });

  // navigator.mediaSession.setActionHandler("volumedown", (event) => {
  //   const logEl = document.getElementById("log");
  //   logEl.innerHTML += "volumeup";
  // });
}

// add voice controls
// ===================
function addVoiceControls() {
}

// printing messages on screen for debugging
function printLogMsg(message) {
  const logEl = document.getElementById("log");
  logEl.innerHTML += `${message}<br>`;
  logEl.scrollTop = logEl.scrollHeight;
  console.debug(message);
}

init();
